@@ -1,11 +1,11 @@
-import anndata
+
 import inspect
 from functools import wraps
 import numpy as np
 import logging
-from tqdm import tqdm
 from scipy.sparse import isspmatrix
 from scipy import sparse
+from anndata import AnnData
 logger = logging.getLogger(__name__)
 
 def return_output(adata, output):
@@ -37,7 +37,13 @@ def wraps_functional(
         func.__signature__ = inspect.Signature(list(getter_signature.values()))
         
         @wraps(func)
-        def _run(adata, **kwargs):
+        def _run(adata, *args, **kwargs):
+            
+            if not isinstance(adata, AnnData):
+                raise TypeError('First argument of this function must be an AnnData object')
+
+            if not len(args) == 0:
+                raise TypeError('Positional arguments are not allowed for this function')
 
             getter_kwargs = {
                 arg : kwargs[arg]
@@ -99,7 +105,13 @@ def wraps_modelfunc(
         func.__signature__ = inspect.Signature(list(getter_signature.values()))
         
         @wraps(func)
-        def _run(self, adata, **kwargs):
+        def _run(self, adata, *args, **kwargs):
+
+            if not isinstance(adata, AnnData):
+                raise TypeError('First argument of this function must be an AnnData object')
+
+            if not len(args) == 0:
+                raise TypeError('Positional arguments are not allowed for this function')
 
             getter_kwargs = {
                 arg : kwargs[arg]
@@ -236,4 +248,9 @@ def add_layer(adata, output, add_layer = 'imputed', sparse = False):
 def add_obsm(adata, output,*,add_key):
 
     logger.info('Added key to obsm: ' + str(add_key))
-    adata.obsm[add_key] = output
\ No newline at end of file
+    adata.obsm[add_key] = output
+
+def add_varm(adata, output,*,add_key):
+
+    logger.info('Added key to varm: ' + str(add_key))
+    adata.varm[add_key] = output
\ No newline at end of file
