@@ -1,5 +1,5 @@
-from mira.plots.chromatin_differential_plot import plot_chromatin_differential
+from mira.plots.chromatin_differential_plot import plot_chromatin_differential, plot_scatter_chromatin_differential
 from mira.plots.streamplot import plot_stream
 from mira.plots.factor_influence_plot import plot_factor_influence, compare_driver_TFs_plot
 from mira.plots.enrichment_plot import plot_enrichments
-from mira.plots.eigengap_plot import plot_eigengap
\ No newline at end of file
+from mira.plots.eigengap_plot import plot_eigengap
