@@ -1,25 +1,21 @@
 
 import numpy as np
-from numpy.core.numeric import ones
 import pyro
 import pyro.distributions as dist
 import torch
 import torch.nn as nn
 import torch.nn.functional as F
-import torch.distributions.constraints as constraints
-from tqdm import tqdm
+from tqdm.auto import tqdm
 import warnings
 from sklearn.preprocessing import scale
 from scipy import sparse
 from scipy.stats import fisher_exact
 from scipy.sparse import isspmatrix
-from mira.topic_model.base import BaseModel, get_fc_stack
-import matplotlib.pyplot as plt
+from mira.topic_model.base import BaseModel, get_fc_stack, logger
 from pyro.contrib.autoname import scope
 from pyro import poutine
 from sklearn.preprocessing import scale
 import mira.adata_interface.core as adi
-import mira.adata_interface.topic_model as tmi
 import mira.adata_interface.regulators as ri
 from mira.plots.factor_influence_plot import plot_factor_influence
 
@@ -83,6 +79,7 @@ class DANEncoder(nn.Module):
 
         return theta_loc, theta_scale
 
+
     def topic_comps(self, idx, read_depth):
         theta = self.forward(idx, read_depth)[0]
         theta = theta.exp()/theta.exp().sum(-1, keepdim = True)
@@ -94,34 +91,6 @@ class AccessibilityTopicModel(BaseModel):
 
     encoder_model = DANEncoder
 
-    @classmethod
-    def _load_old_model(cls, filename, varnames):
-
-        old_model = torch.load(filename)
-
-        params = old_model['params'].copy()
-        params['num_topics'] = params.pop('num_modules')
-
-        if params['highly_variable'] is None:
-            params['highly_variable'] = np.ones(len(varnames)).astype(bool)
-        
-        fit_params = {}
-        fit_params['num_exog_features'] = len(params['features'])
-        fit_params['num_endog_features'] = params['highly_variable'].sum()
-        fit_params['highly_variable'] = params.pop('highly_variable')
-        fit_params['features'] = varnames
-        params.pop('features')
-        
-        model = cls(**params)
-        model._set_weights(fit_params, old_model['model']['weights'])
-
-        if 'pi' in old_model['model']:
-            model.residual_pi = old_model['model']['pi']
-        
-        model.enrichments = {}
-        
-        return model
-
     @property
     def peaks(self):
         return self.features
@@ -132,7 +101,7 @@ class AccessibilityTopicModel(BaseModel):
         
         with pyro.plate("cells", endog_features.shape[0]):
 
-            with poutine.scale(None, anneal_factor):
+            with poutine.scale(None, anneal_factor/self.reconstruction_weight):
                 theta = pyro.sample(
                     "theta", dist.LogNormal(theta_loc, theta_scale).to_event(1)
                 )
@@ -153,7 +122,7 @@ class AccessibilityTopicModel(BaseModel):
             
             theta_loc, theta_scale = self.encoder(endog_features, read_depth)
 
-            with poutine.scale(None, anneal_factor):
+            with poutine.scale(None, anneal_factor/self.reconstruction_weight):
                     
                 theta = pyro.sample(
                     "theta", dist.LogNormal(theta_loc, theta_scale).to_event(1)
@@ -169,7 +138,7 @@ class AccessibilityTopicModel(BaseModel):
         assert(len(X.shape) == 2)
         assert(X.shape[1] == expected_width)
         
-        assert(np.isclose(X.data.astype(np.int64), X.data, 1e-2).all()), 'Input data must be raw transcript counts, represented as integers. Provided data contains non-integer values.'
+        assert(np.isclose(X.data.astype(np.uint16), X.data, 1e-2).all()), 'Input data must be raw transcript counts, represented as integers. Provided data contains non-integer values.'
 
         X.data = np.ones_like(X.data)
 
@@ -190,33 +159,43 @@ class AccessibilityTopicModel(BaseModel):
 
         dense_matrix = np.vstack(dense_matrix)
         
-        return dense_matrix.astype(np.int64)
+        return dense_matrix
 
-    
-    def _preprocess_endog(self, X, read_depth):
+
+    def get_endog_fn(self):
+
+        def preprocess_endog(X):
         
-        return torch.tensor(
-            self._get_padded_idx_matrix(self._binarize_matrix(X, self.num_endog_features)), 
-            requires_grad = False
-        ).to(self.device)
+            return self._get_padded_idx_matrix(
+                    self._binarize_matrix(X, self.num_endog_features)).astype(np.int32)
 
-
-    def _preprocess_exog(self, X):
+        return preprocess_endog
+                   
+    def get_exog_fn(self):
         
-        return torch.tensor(
-            self._get_padded_idx_matrix(self._binarize_matrix(X, self.num_exog_features)), 
-            requires_grad = False
-        ).to(self.device)
+        def preprocess_exog(X):
 
+            return self._get_padded_idx_matrix(
+                    self._binarize_matrix(X, self.num_exog_features)
+                    ).astype(np.int64)
+
+        return preprocess_exog
+
+    def get_dataloader(self, dataset, training = False):
+        
+        if self.dataset_loader_workers == 0:
+            logger.warn('Dataset loader allocated no extra workers. This will slow down training by ~50%.')
+
+        return super().get_dataloader(dataset, training = training)
 
     def _argsort_peaks(self, topic_num):
         assert(isinstance(topic_num, int) and topic_num < self.num_topics and topic_num >= 0)
         return np.argsort(self._score_features()[topic_num, :])
 
-
     def rank_peaks(self, topic_num):
         return self.peaks[self._argsort_peaks(topic_num)]
 
+
     def _validate_hits_matrix(self, hits_matrix):
         assert(isspmatrix(hits_matrix))
         assert(len(hits_matrix.shape) == 2)
@@ -253,8 +232,12 @@ class AccessibilityTopicModel(BaseModel):
         
         Examples
         --------
-        >>> mira.tl.get_motif_hits_in_peaks(atac_data, genome_fasta = '~/genome.fa')
-        >>> atac_model.get_enriched_TFs(atac_data, topic_num = 10)
+
+        .. code-block:: python
+
+            >>> mira.tl.get_motif_hits_in_peaks(atac_data, genome_fasta = '~/genome.fa')
+            >>> atac_model.get_enriched_TFs(atac_data, topic_num = 10)
+
         '''
 
         assert(isinstance(top_quantile, float) and top_quantile > 0 and top_quantile < 1)
@@ -287,6 +270,30 @@ class AccessibilityTopicModel(BaseModel):
     @adi.wraps_modelfunc(ri.fetch_factor_hits_and_latent_comps, ri.make_motif_score_adata,
         ['metadata','hits_matrix','topic_compositions'])
     def get_motif_scores(self, batch_size=512,*, metadata, hits_matrix, topic_compositions):
+        '''
+        Get motif scores for each cell based on the probability of sampling a motif
+        from the posterior distribution over accessible sites in a cell.
+
+        Parameters
+        ----------
+
+        adata : anndata.AnnData
+            AnnData of accessibility features, annotated with TF binding using
+            mira.tl.get_motif_hits_in_peaks or mira.tl.get_ChIP_hits_in_peaks.
+        batch_size : int>0, default=512
+            Minibatch size to calculate posterior distribution over accessible
+            regions. Only affects the amount of memory used.
+        factor_type : str, 'motifs' or 'chip', default = 'motifs'
+            Which factor type to use for enrichment.
+
+        Returns
+        -------
+
+        motif_scores : anndata.AnnData of shape (n_cells, n_factors)
+            AnnData object. For each cell from the original adata, gives
+            score for each motif/ChIP factor. 
+
+        '''
 
         hits_matrix = self._validate_hits_matrix(hits_matrix)
     
@@ -303,6 +310,33 @@ class AccessibilityTopicModel(BaseModel):
 
 
     def get_enrichments(self, topic_num, factor_type = 'motifs'):
+        '''
+        Returns TF enrichments for a certain topic.
+
+        Parameters
+        ----------
+        topic_num : int
+            For which topic to return results
+        factor_type : str, 'motifs' or 'chip', default = 'motifs'
+            Which factor type to use for enrichment
+
+        Returns
+        -------
+        
+        topic_enrichments : list[dict]
+            For each record, gives a dict of 
+            {'factor_id' : <id>,
+            'name' : <name>,
+            'parsed_name' : <name used for expression lookup>,
+            'pval' : <pval>,
+            'test_statistic' : <statistic>}
+
+        Raises
+        ------
+
+        KeyError : if *get_enriched_TFs* was not yet run for the given topic.
+
+        '''
         try:
             return self.enrichments[(factor_type, topic_num)]
         except KeyError:
@@ -315,6 +349,86 @@ class AccessibilityTopicModel(BaseModel):
         ax = None, figsize = (8,8), legend_label = '', show_legend = True, fontsize = 13, 
         pval_threshold = (1e-50, 1e-50), na_color = 'lightgrey',
         color = 'grey', label_closeness = 3, max_label_repeats = 3, show_factor_ids = False):
+        '''
+        It is often useful to contrast topic enrichments in order to
+        understand which factors' influence is unique to certain
+        cell states. Topics may be enriched for constitutively-active
+        transcription factors, so comparing two similar topics to find
+        the factors that are unique to each elucidates the dynamic
+        aspects of regulation between states.
+
+        This function contrasts the enrichments of two topics.
+
+        Parameters
+        ----------
+
+        topic1, topic2 : int
+            Which topics to compare.
+        factor_type : str, 'motifs' or 'chip', default = 'motifs'
+            Which factor type to use for enrichment.
+        label_factors : list[str], np.ndarray[str], None; default=None
+            List of factors to label. If not provided, will label all
+            factors that meet the p-value thresholds.
+        hue : dict[str : {str, float}] or None
+            If provided, colors the factors on the plot. The keys of the dict
+            must be the names of transcription factors, and the values are
+            the associated data to map to colors. The values may be 
+            categorical, e.g. cluster labels, or scalar, e.g. expression
+            values. TFs not provided in the dict are colored as *na_color*.
+        palette : str, list[str], or None; default = None
+            Palette of plot. Default of None will set `palette` to the style-specific default.
+        hue_order : list[str] or None, default = None
+            Order to assign hues to features provided by `data`. Works similarly to
+            hue_order in seaborn. User must provide list of features corresponding to 
+            the order of hue assignment. 
+        ax : matplotlib.pyplot.axes, deafult = None
+            Provide axes object to function to add streamplot to a subplot composition,
+            et cetera. If no axes are provided, they are created internally.
+        figsize : tuple(float, float), default = (8,8)
+            Size of figure
+        legend_label : str, None
+            Label for legend.
+        show_legend : boolean, default=True
+            Show figure legend.
+        fontsize : int>0, default=13
+            Fontsize of TF labels on plot.
+        pval_threshold : tuple[float, float], default=(1e-50, 1e-50)
+            Threshold below with TFs will not be labeled on plot. The first and
+            second positions relate p-value with respect to topic 1 and topic 2.
+        na_color : str, default='lightgrey'
+            Color for TFs with no provided *hue*
+        color : str, default='grey'
+            If *hue* not provided, colors all points on plot this color.
+        label_closeness : int>0, default=3
+            Closeness of TF labels to points on plot. When *label_closeness* is high,
+            labels are forced to be very close to points.
+        max_label_repeats : boolean, default=3
+            Some TFs have multiple ChIP samples or Motif PWMs. For these factors,
+            label the top *max_label_repeats* examples. This prevents clutter when
+            many samples for the same TF are close together. The rank of the sample
+            for each TF is shown in the label as "<TF name> (<rank>)".
+
+        Returns
+        -------
+
+        matplotlib.pyplot.axes
+
+        Examples
+        --------
+
+        .. code-block :: python
+
+            >>> label = ['LEF1','HOXC13','MEOX2','DLX3','BACH2','RUNX1', 'SMAD2::SMAD3']
+            >>> atac_model.plot_compare_topic_enrichments(23, 17,
+            ...     label_factors = label, 
+            ...     color = 'lightgrey',
+            ...     fontsize=20, label_closeness=5, 
+            ... )
+
+        .. image:: /_static/mira.topics.AccessibilityModel.plot_compare_topic_enrichments.svg
+            :width: 300
+
+        '''
 
         m1 = self.get_enrichments(topic_1, factor_type)
         m2 = self.get_enrichments(topic_2, factor_type)        
